function [luminanceContrast, textureContrast, salienceMap] = stimulusSaliency(img,quiet)
% computes saliency map based on luminance and texture contrast:
%   Parkhurst D.J. and Niebur E., 
%   Texture contrast attracts overt visual attention in natural scenes
%   Vol. 19, pp. 783-789, 2004
%
% J.Filip, 18/12/2009

imgOrig = img;

% --- generate gaussian pyramid in image luminance -----------------------
% if(~quiet)
%     figure(1);
% end;
ReductionFactor = 1;
KernelSize = 7;
Sigma = 1;
n = 6;

means = zeros(1,n);
stds = zeros(1,n);

isY = size(img,1);
isX = size(img,2);
gausPyr = zeros(isY,isX,n);
gausPyrSizesY = zeros(n);
gausPyrSizesX = zeros(n);
for i=1:n
    if(~quiet)
        subplot(3,n,i);
    end;
    mi = min(min(img));
    ma = max(max(img));
    if(~quiet)
        imshow(img,[0 255]);
    end;
    
    % storing gaussian pyramid
    gausPyr(1:size(img,1),1:size(img,2),i) = img(:,:);
    gausPyrSizesY(i) = size(img,1);
    gausPyrSizesX(i) = size(img,2);

    imgRes = gaussianPyramid(img, ReductionFactor, KernelSize, Sigma);
    
%     ma = max(max(imgRes));
%     mi = min(min(imgRes));
%     imgRes = 255*imgRes;
%     if(~quiet)
%         subplot(3,n,n+i);
%     end;
%     scale = is/size(imgRes,1);
%     auxImg = imresize(imgRes,size(img),'Method','bicubic');
%     if(~quiet)
%         imshow(auxImg,[0 255]);
%     end;
%     means(i) = mean(mean(auxImg));
%     stds(i) = var(var(auxImg));
% 
%     if(~quiet)
%         subplot(3,n,2*n+i);
%     end;
%     diff = double(img)-double(auxImg);
%     if(~quiet)
%        imshow(diff,[-64 64]);
%     end;
    
    img = imgRes;
end;

%--- general params ------------------------------
mapSizeY = round(isY/1.0); % size of final salience map
mapSizeX = round(isX/1.0); % size of final salience map
interp = 'bicubic'; % interpolation type for pyramid levels resampling to 'mapSize'

%--- compute luminance contrast -------------------------------------
surr = 11;
r_c = 0.5;
r_s = 2.5;
beta = 2;
s = 5;

% surr = 15;
% r_c = 0.5;
% r_s = 2.5;
% beta = 2;
% s = 5;

% define gaussian kernels for center and surround
h_c = fspecial('gaussian',[surr surr],r_c);
h_s = fspecial('gaussian',[surr surr],r_s);

csPyrOnOff = zeros(isY,isX,n,'double');
csPyrOffOn = zeros(isY,isX,n,'double');
csPyr_max = zeros(n,4,'double');
for i=1:n
    L_c = imfilter(gausPyr(1:gausPyrSizesY(i),1:gausPyrSizesX(i),i),h_c);
    L_s = imfilter(gausPyr(1:gausPyrSizesY(i),1:gausPyrSizesX(i),i),h_s);
    csPyrOnOff(1:size(L_c,1),1:size(L_c,2),i) = pos(L_c - L_s);
    csPyrOffOn(1:size(L_c,1),1:size(L_c,2),i) = pos(L_s - L_c);
    % divisive inhibition
    aux = csPyrOnOff(1:size(L_c,1),1:size(L_c,2),i);
    csPyrOnOff(1:size(L_c,1),1:size(L_c,2),i) = aux.^beta./(aux.^beta+s.^beta);
    aux = csPyrOffOn(1:size(L_c,1),1:size(L_c,2),i);
    csPyrOffOn(1:size(L_c,1),1:size(L_c,2),i) = aux.^beta./(aux.^beta+s.^beta);
    csPyr_max(i,1) = max(max(csPyrOnOff(:,:,i)));
    csPyr_max(i,2) = max(max(csPyrOffOn(:,:,i)));
end;

if(~quiet)
    figure(2);
end;
luminanceContrast = zeros(mapSizeY,mapSizeX,'double');
for i=1:n
    if(~quiet)
        subplot(2,n,i);
        imshow(csPyrOnOff(1:gausPyrSizes(i),1:gausPyrSizes(i),i),[0 max(csPyr_max(:,1))]);
    end;
    luminanceContrast = luminanceContrast + imresize(csPyrOnOff(1:gausPyrSizesY(i),1:gausPyrSizesX(i),i),[floor(mapSizeY),floor(mapSizeX)],'Method','bicubic');

    if(~quiet)
        subplot(2,n,n+i);
        imshow(csPyrOffOn(1:gausPyrSizes(i),1:gausPyrSizes(i),i),[0 max(csPyr_max(:,2))]);
    end;
    luminanceContrast = luminanceContrast + imresize(csPyrOffOn(1:gausPyrSizesY(i),1:gausPyrSizesX(i),i),[floor(mapSizeY),floor(mapSizeX)],'Method',interp);
end;

%--- compute texture contrast -------------------------------------
k = 1;

% define gaussian kernels for center and surround
h_c = fspecial('gaussian',[surr surr],k*r_c);
h_s = fspecial('gaussian',[surr surr],k*r_s);

csPyrOnOnOff = zeros(isY,isX,n,'double');
csPyrOffOnOff = zeros(isY,isX,n,'double');
csPyrOnOffOn = zeros(isY,isX,n,'double');
csPyrOffOffOn = zeros(isY,isX,n,'double');
for i=1:n
    L_c = imfilter(csPyrOnOff(1:gausPyrSizesY(i),1:gausPyrSizesX(i),i),h_c);
    L_s = imfilter(csPyrOnOff(1:gausPyrSizesY(i),1:gausPyrSizesX(i),i),h_s);
    csPyrOnOnOff(1:size(L_c,1),1:size(L_c,2),i) = pos(L_c - L_s);
    csPyrOnOffOn(1:size(L_c,1),1:size(L_c,2),i) = pos(L_s - L_c);
    csPyr_max(i,1) = max(max(csPyrOnOnOff(:,:,i)));
    csPyr_max(i,2) = max(max(csPyrOnOffOn(:,:,i)));
    L_c = imfilter(csPyrOffOn(1:gausPyrSizesY(i),1:gausPyrSizesX(i),i),h_c);
    L_s = imfilter(csPyrOffOn(1:gausPyrSizesY(i),1:gausPyrSizesX(i),i),h_s);
    csPyrOffOnOff(1:size(L_c,1),1:size(L_c,2),i) = pos(L_c - L_s);
    csPyrOffOffOn(1:size(L_c,1),1:size(L_c,2),i) = pos(L_s - L_c);
    csPyr_max(i,3) = max(max(csPyrOffOnOff(:,:,i)));
    csPyr_max(i,4) = max(max(csPyrOffOffOn(:,:,i)));
end;

if(~quiet)
    figure(3);  
end;
textureContrast = zeros(floor(mapSizeY),floor(mapSizeX),'double');
for i=1:n
    if(~quiet)
        subplot(4,n,i);
        imshow(csPyrOnOnOff(1:gausPyrSizes(i),1:gausPyrSizes(i),i),[0 max(csPyr_max(:,1))]);
    end;
    textureContrast = textureContrast + imresize(csPyrOnOnOff(1:gausPyrSizesY(i),1:gausPyrSizesX(i),i),[floor(mapSizeY),floor(mapSizeX)],'Method',interp);

    if(~quiet)
        subplot(4,n,n+i);
        imshow(csPyrOnOffOn(1:gausPyrSizes(i),1:gausPyrSizes(i),i),[0 max(csPyr_max(:,2))]);
    end;
    textureContrast = textureContrast + imresize(csPyrOnOffOn(1:gausPyrSizesY(i),1:gausPyrSizesX(i),i),[floor(mapSizeY),floor(mapSizeX)],'Method',interp);

    if(~quiet)
        subplot(4,n,2*n+i);
        imshow(csPyrOffOnOff(1:gausPyrSizes(i),1:gausPyrSizes(i),i),[0 max(csPyr_max(:,3))]);
    end;
    textureContrast = textureContrast + imresize(csPyrOffOnOff(1:gausPyrSizesY(i),1:gausPyrSizesX(i),i),[floor(mapSizeY),floor(mapSizeX)],'Method',interp);

    if(~quiet)
        subplot(4,n,3*n+i);
        imshow(csPyrOffOffOn(1:gausPyrSizes(i),1:gausPyrSizes(i),i),[0 max(csPyr_max(:,4))]);
    end;
    textureContrast = textureContrast + imresize(csPyrOffOffOn(1:gausPyrSizesY(i),1:gausPyrSizesX(i),i),[floor(mapSizeY),floor(mapSizeX)],'Method',interp);
    
end;

w = 10;
salienceMap = luminanceContrast + w*textureContrast; % weighting contrast images

if(quiet==0)
    %--- plotting salience results ------------------------------------
    figure(4);
    subplot(2,2,1);
    imshow(imgOrig,[0 max(max(imgOrig))]);
    title('Stimulus luminance');
    subplot(2,2,2);
    imshow(salienceMap,[0, max(max(salienceMap))]);
    title('Salience Map');
    xlabel(sprintf('Range %f',max(max(salienceMap))))
    subplot(2,2,3);
    imshow(luminanceContrast,[0, max(max(luminanceContrast))]);
    title('Luminance Contrast');
    xlabel(sprintf('Range %f',max(max(luminanceContrast))))
    subplot(2,2,4);
    imshow(textureContrast,[0, max(max(textureContrast))]);
    title('Texture Contrast');
    xlabel(sprintf('Range %f',max(max(textureContrast))))
end;
